﻿namespace AppleFactory {
	
	// Library

	class Apple {
		public int Energy { get; init; }

		public void Eat() {
            Console.WriteLine($"Apple eaten, player got {Energy} units of energy.");
        }
	}

	class AppleFactory {
		private int _appleEnergy;

		public AppleFactory(int appleEnergy) {
			_appleEnergy = appleEnergy;
		}

		public Apple Create() {
			return new Apple { Energy = _appleEnergy };
		}
	}

	class Tree {
		private AppleFactory _factory;

		public Tree(AppleFactory factory) {
			_factory = factory;
		}

		public List<Apple> Harvest() {
			return new List<Apple> {
				_factory.Create(),
				_factory.Create(),
				_factory.Create()
			};
		}
	}

	// Application
	
	internal class Program {
		static void Main(string[] args) {

			var goodAppleFactory = new AppleFactory(10);
			var goodTree = new Tree(goodAppleFactory);
			HarvestAndEatAllApples(goodTree);

			var badAppleFactory = new AppleFactory(2);
			var badTree = new Tree(badAppleFactory);
			HarvestAndEatAllApples(badTree);
		}

		static void HarvestAndEatAllApples(Tree tree) {
			var apples = tree.Harvest();
			foreach (Apple apple in apples) {
				apple.Eat();
			}
            Console.WriteLine();
        }
	}
}