﻿namespace AppleFactory {

	// Library

	class Apple {
		public int Energy { get; init; }

		public virtual void Eat() {
			Console.WriteLine($"Apple eaten, player got {Energy} units of energy.");
		}
	}

	class GoldenApple : Apple {
		public override void Eat() {
			base.Eat();
			Console.WriteLine("Golden apple increased player's luck.");
		}
	}

	class AppleFactory {
		private int _applesDispenced = 0;
		private int _appleEnergy;

		public AppleFactory(int appleEnergy) {
			_appleEnergy = appleEnergy;
		}

		public Apple Create() {
			if ((_applesDispenced++) % 2 == 0) {
				return new GoldenApple { Energy = _appleEnergy };
			} else {
				return new Apple { Energy = _appleEnergy };
			}
		}
	}

	class Tree {
		private AppleFactory _factory;

		public Tree(AppleFactory factory) {
			_factory = factory;
		}

		public List<Apple> Harvest() {
			return new List<Apple> {
				_factory.Create(),
				_factory.Create(),
				_factory.Create()
			};
		}
	}

	// Application

	internal class Program {
		static void Main(string[] args) {

			var goodAppleFactory = new AppleFactory(10);
			var goodTree = new Tree(goodAppleFactory);
			HarvestAndEatAllApples(goodTree);

			var badAppleFactory = new AppleFactory(2);
			var badTree = new Tree(badAppleFactory);
			HarvestAndEatAllApples(badTree);
		}

		static void HarvestAndEatAllApples(Tree tree) {
			var apples = tree.Harvest();
			foreach (Apple apple in apples) {
				apple.Eat();
			}
			Console.WriteLine();
		}
	}
}